﻿Option Strict On
Option Explicit On

Imports System.IO
Imports System.Text
Imports System.Runtime.InteropServices
Imports System.Drawing.Imaging

Public Class PreviewHandler

#Region "API"
    <DllImport("Ole32.dll", EntryPoint:="CoCreateInstance")>
    Private Shared Function CoCreateInstance(<[In]> ByRef rclsid As Guid,
                                             <[In]> pUnkOuter As IntPtr,
                                             <[In]> dwClsContext As Integer,
                                             <[In]> ByRef riid As Guid,
                                             <Out> ByRef ppv As IntPtr) As Integer
    End Function

    <DllImport("Shlwapi.dll", EntryPoint:="AssocQueryStringW")>
    <PreserveSig> Private Shared Function AssocQueryStringW(<[In]> flags As ASSOCF,
                                                            <[In]> str As ASSOCSTR,
                                                            <[In], MarshalAs(UnmanagedType.LPWStr)> pszAssoc As String,
                                                            <[In], MarshalAs(UnmanagedType.LPWStr)> pszExtra As String,
                                                            <[In], Out, MarshalAs(UnmanagedType.LPWStr)> pszOut As StringBuilder,
                                                            <Out> ByRef pcchOut As Integer) As Integer
    End Function

    <DllImport("Shlwapi.dll", EntryPoint:="SHCreateStreamOnFileEx")>
    <PreserveSig> Private Shared Function SHCreateStreamOnFileEx(<[In], MarshalAs(UnmanagedType.LPWStr)> pszFile As String,
                                                                 <[In]> grfMode As STGM,
                                                                 <[In]> dwAttributes As Integer,
                                                                 <[In], MarshalAs(UnmanagedType.Bool)> fCreate As Boolean,
                                                                 <[In]> pstmTemplate As IntPtr,
                                                                 <Out> ByRef ppstm As IntPtr) As Integer
    End Function

    <DllImport("Shell32.dll", EntryPoint:="SHCreateItemFromParsingName")>
    <PreserveSig> Private Shared Function SHCreateItemFromParsingName(<[In], MarshalAs(UnmanagedType.LPWStr)> pszPath As String,
                                                                      <[In]> pbc As IntPtr,
                                                                      <[In], MarshalAs(UnmanagedType.LPStruct)> riid As Guid,
                                                                      <Out> ByRef pUnk As IntPtr) As Integer
    End Function

    <DllImport("Gdi32.dll", EntryPoint:="DeleteObject")>
    Private Shared Function DeleteObject(<[In]> hgdiobj As IntPtr) As <MarshalAs(UnmanagedType.Bool)> Boolean
    End Function
#End Region

#Region "Enum"

    Private Enum WTS_ALPHATYPE As Integer
        WTSAT_UNKNOWN = &H0&
        WTSAT_RGB = &H1&
        WTSAT_ARGB = &H2&
    End Enum

    Private Enum STGM As Integer
        STGM_FAILIFTHERE = &H0&
        STGM_DIRECT = &H0&
        STGM_READ = &H0&
        STGM_WRITE = &H1&
        STGM_READWRITE = &H2&
        STGM_SHARE_EXCLUSIVE = &H10&
        STGM_SHARE_DENY_WRITE = &H20&
        STGM_SHARE_DENY_READ = &H30&
        STGM_SHARE_DENY_NONE = &H40&
        STGM_CREATE = &H1000&
        STGM_TRANSACTED = &H10000&
        STGM_CONVERT = &H20000&
        STGM_PRIORITY = &H40000&
        STGM_NOSCRATCH = &H100000&
        STGM_NOSNAPSHOT = &H200000&
        STGM_DIRECT_SWMR = &H400000&
        STGM_SIMPLE = &H8000000&
        STGM_DELETEONRELEASE = &H4000000&
    End Enum

    Private Enum SIIGBF As Integer
        SIIGBF_RESIZETOFIT = &H0&
        SIIGBF_BIGGERSIZEOK = &H1&
        SIIGBF_MEMORYONLY = &H2&
        SIIGBF_ICONONLY = &H4&
        SIIGBF_THUMBNAILONLY = &H8&
        SIIGBF_INCACHEONLY = &H10&
        SIIGBF_CROPTOSQUARE = &H20&
        SIIGBF_WIDETHUMBNAILS = &H40&
        SIIGBF_ICONBACKGROUND = &H80&
        SIIGBF_SCALEUP = &H100&
    End Enum

    Private Enum ASSOCF As Integer
        ASSOCF_NONE = &H0&
        ASSOCF_INIT_NOREMAPCLSID = &H1&
        ASSOCF_INIT_BYEXENAME = &H2&
        ASSOCF_OPEN_BYEXENAME = &H2&
        ASSOCF_INIT_DEFAULTTOSTAR = &H4&
        ASSOCF_INIT_DEFAULTTOFOLDER = &H8&
        ASSOCF_NOUSERSETTINGS = &H10&
        ASSOCF_NOTRUNCATE = &H20&
        ASSOCF_VERIFY = &H40&
        ASSOCF_REMAPRUNDLL = &H80&
        ASSOCF_NOFIXUPS = &H100&
        ASSOCF_IGNOREBASECLASS = &H200&
        ASSOCF_INIT_IGNOREUNKNOWN = &H400&
        ASSOCF_INIT_FIXED_PROGID = &H800&
        ASSOCF_IS_PROTOCOL = &H1000&
        ASSOCF_INIT_FOR_FILE = &H2000&
    End Enum

    Private Enum ASSOCSTR As Integer
        ASSOCSTR_COMMAND = &H1&
        ASSOCSTR_EXECUTABLE = &H2&
        ASSOCSTR_FRIENDLYDOCNAME = &H3&
        ASSOCSTR_FRIENDLYAPPNAME = &H4&
        ASSOCSTR_NOOPEN = &H5&
        ASSOCSTR_SHELLNEWVALUE = &H6&
        ASSOCSTR_DDECOMMAND = &H7&
        ASSOCSTR_DDEIFEXEC = &H8&
        ASSOCSTR_DDEAPPLICATION = &H9&
        ASSOCSTR_DDETOPIC = &HA&
        ASSOCSTR_INFOTIP = &HB&
        ASSOCSTR_QUICKTIP = &HC&
        ASSOCSTR_TILEINFO = &HD&
        ASSOCSTR_CONTENTTYPE = &HE&
        ASSOCSTR_DEFAULTICON = &HF&
        ASSOCSTR_SHELLEXTENSION = &H10&
        ASSOCSTR_DROPTARGET = &H11&
        ASSOCSTR_DELEGATEEXECUTE = &H12&
        ASSOCSTR_SUPPORTED_URI_PROTOCOLS = &H13&
        ASSOCSTR_PROGID = &H14&
        ASSOCSTR_APPID = &H15&
        ASSOCSTR_APPPUBLISHER = &H16&
        ASSOCSTR_APPICONREFERENCE = &H17&
        ASSOCSTR_MAX = &H18&
    End Enum

    Private Enum IEI_FLAGS As Integer
        IEI_PRIORITY_NORMAL = &H0&
        IEI_PRIORITY_MIN = &H1&
        IEI_PRIORITY_MAX = &H2&
    End Enum

    Private Enum IEIFLAG_FLAGS As Integer
        IEIFLAG_OFFLINE = &H8&
        IEIFLAG_SCREEN = &H20&
        IEIFLAG_ORIGSIZE = &H40&
        IEIFLAG_QUALITY = &H200&
        IEIFLAG_REFRESH = &H400&
    End Enum
#End Region

#Region "Const"
    Private Const S_OK As Integer = 0
    Private Const MAX_PATH As Integer = 260
    Private Const CLSCTX_LOCAL_SERVER As Integer = 4
    Private Const CLSCTX_INPROC_SERVER As Integer = 1
    Private Const FILE_ATTRIBUTE_NORMAL As Integer = 0

    Private Const IID_IOleWindow As String = "00000114-0000-0000-c000-000000000046"
    Private Const IID_IShellItem As String = "43826d1e-e718-42ee-bc55-a1e261c37bfe"
    Private Const IID_IExtractImage As String = "bb2e617c-0920-11d1-9a0b-00c04fc2d6c1"
    Private Const IID_IPreviewHandler As String = "8895b1c6-b41f-4c1c-a562-0d564250836f"
    Private Const IID_IThumbnailProvider As String = "e357fccd-a995-4576-b01f-234630154e96"
    Private Const IID_IInitializeWithFile As String = "b7d14566-0509-4cce-a71f-0a554233bd9b"
    Private Const IID_IInitializeWithItem As String = "7f73be3f-fb79-493c-a6c7-7ee14e245841"
    Private Const IID_IInitializeWithStream As String = "b824b49d-22ac-4161-ac8a-9916e8fa3f7f"
    Private Const IID_IPreviewHandlerVisuals As String = "196bf9a5-b346-4ef0-aa1e-5dcdb76768b1"
    Private Const IID_IShellItemImageFactory As String = "bcc18b79-ba16-442f-80c4-8a59c30c463b"
#End Region

#Region "Class"
    Protected Overrides Sub Finalize()
        DeletePreview()
        MyBase.Finalize()
    End Sub
#End Region

#Region "Variable"
    Private m_PreviewSize As New Size
    Private m_TextColor As Integer = 0
    Private m_BackgroundColor As Integer = 0
    Private m_PreviewWindow As Control = Nothing
    Private m_IPreviewHandler As Object = Nothing
    Private m_PreviewFile As String = String.Empty
#End Region

#Region "Public Functions"
    Public Function ShowPreview() As Boolean
        Return ShowPreviewIntern()
    End Function

    Public Function ShowPreview(sPreviewFile As String) As Boolean
        PreviewFile = sPreviewFile
        Return ShowPreviewIntern()
    End Function

    Public Function ClosePreview() As Boolean
        Return DeletePreview()
    End Function

    Public Function ResizePreview(NewSize As Size) As Boolean
        Dim bolRet As Boolean = False
        m_PreviewSize = NewSize
        If m_IPreviewHandler IsNot Nothing Then
            If CType(m_IPreviewHandler, IPreviewHandler).SetRect(New Rectangle(0, 0, m_PreviewSize.Width, m_PreviewSize.Height)) = S_OK Then
                bolRet = True
            End If
        Else
            bolRet = ShowPreviewIntern()
        End If
        Return bolRet
    End Function

#End Region

#Region "Private Funtions"
    Private Function ShowPreviewIntern() As Boolean

        If Not File.Exists(m_PreviewFile) Then Return False

        If m_PreviewWindow Is Nothing Then Return False

        If m_PreviewWindow.BackgroundImage IsNot Nothing Then
            m_PreviewWindow.BackgroundImage.Dispose()
            m_PreviewWindow.BackgroundImage = Nothing
        End If

        If m_IPreviewHandler IsNot Nothing Then
            If TypeOf m_IPreviewHandler Is IPreviewHandler Then CType(m_IPreviewHandler, IPreviewHandler).Unload()
            If Marshal.ReleaseComObject(m_IPreviewHandler) = 0 Then
                m_IPreviewHandler = Nothing
            End If
        End If

        Dim bolRet As Boolean = False
        Dim pIPreviewHandler As IntPtr
        Dim strHandlerClsid As String = String.Empty
        Dim PrevSize As Size = If(m_PreviewSize.IsEmpty, New Size(m_PreviewWindow.Width, m_PreviewWindow.Height), m_PreviewSize)
        Dim PrevRect As New Rectangle(0, 0, PrevSize.Width, PrevSize.Height)

        If GetHandlerClsidFromFile(m_PreviewFile, IID_IPreviewHandler, strHandlerClsid) Then

            pIPreviewHandler = CreateInterface(strHandlerClsid, IID_IPreviewHandler)

            If pIPreviewHandler <> IntPtr.Zero Then
                m_IPreviewHandler = Marshal.GetObjectForIUnknown(pIPreviewHandler)
                Marshal.Release(pIPreviewHandler)
            End If

            If m_IPreviewHandler IsNot Nothing Then

                If InitializePreviewHandler(m_IPreviewHandler, m_PreviewFile) Then

                    If CType(m_IPreviewHandler, IPreviewHandler).SetWindow(m_PreviewWindow.Handle, PrevRect) = S_OK Then

                        If CType(m_IPreviewHandler, IPreviewHandler).SetRect(PrevRect) = S_OK Then

                            If TypeOf m_IPreviewHandler Is IPreviewHandlerVisuals Then
                                CType(m_IPreviewHandler, IPreviewHandlerVisuals).SetBackgroundColor(m_BackgroundColor)
                                CType(m_IPreviewHandler, IPreviewHandlerVisuals).SetTextColor(m_TextColor)
                            End If

                            If CType(m_IPreviewHandler, IPreviewHandler).DoPreview = S_OK Then
                                bolRet = True
                            End If

                        End If

                    End If

                Else

                    If Marshal.ReleaseComObject(m_IPreviewHandler) = 0 Then m_IPreviewHandler = Nothing

                End If

            End If

        End If

        If Not bolRet Then

            If GetHandlerClsidFromFile(m_PreviewFile, IID_IThumbnailProvider, strHandlerClsid) Then

                pIPreviewHandler = CreateInterface(strHandlerClsid, IID_IThumbnailProvider)

                If pIPreviewHandler <> IntPtr.Zero Then
                    m_IPreviewHandler = Marshal.GetObjectForIUnknown(pIPreviewHandler)
                    Marshal.Release(pIPreviewHandler)
                End If

                If m_IPreviewHandler IsNot Nothing Then

                    If InitializePreviewHandler(m_IPreviewHandler, m_PreviewFile) Then

                        Dim hBitmap As IntPtr
                        Dim dwAlpha As WTS_ALPHATYPE

                        If CType(m_IPreviewHandler, IThumbnailProvider).GetThumbnail(Math.Min(PrevSize.Width, PrevSize.Height), hBitmap, dwAlpha) = S_OK Then

                            If hBitmap <> IntPtr.Zero Then
                                If m_PreviewWindow.BackgroundImage IsNot Nothing Then m_PreviewWindow.BackgroundImage.Dispose()
                                m_PreviewWindow.BackgroundImageLayout = ImageLayout.Center
                                m_PreviewWindow.BackgroundImage = GetBitmapFromHBitmap(hBitmap)
                                DeleteObject(hBitmap)
                                bolRet = True
                            End If

                        End If

                    End If

                    If Marshal.ReleaseComObject(m_IPreviewHandler) = 0 Then m_IPreviewHandler = Nothing

                End If

            End If

        End If

        If Not bolRet Then

            If GetHandlerClsidFromFile(m_PreviewFile, IID_IExtractImage, strHandlerClsid) Then

                pIPreviewHandler = CreateInterface(strHandlerClsid, IID_IExtractImage)

                If pIPreviewHandler <> IntPtr.Zero Then
                    m_IPreviewHandler = Marshal.GetObjectForIUnknown(pIPreviewHandler)
                    Marshal.Release(pIPreviewHandler)
                End If

                If m_IPreviewHandler IsNot Nothing Then
                    If InitializePreviewHandler(m_IPreviewHandler, m_PreviewFile) Then
                        Dim sbBuffer As New StringBuilder(MAX_PATH)
                        Dim ePriority As IEI_FLAGS = IEI_FLAGS.IEI_PRIORITY_NORMAL
                        Dim eFlags As IEIFLAG_FLAGS = IEIFLAG_FLAGS.IEIFLAG_QUALITY
                        Dim intColorDepth As Integer = 32
                        If CType(m_IPreviewHandler, IExtractImage).GetLocation(sbBuffer, sbBuffer.Capacity, ePriority,
                                                                               PrevSize, intColorDepth, eFlags) = S_OK Then
                            Dim hBitmap As IntPtr
                            If CType(m_IPreviewHandler, IExtractImage).Extract(hBitmap) = S_OK Then
                                If hBitmap <> IntPtr.Zero Then
                                    If m_PreviewWindow.BackgroundImage IsNot Nothing Then m_PreviewWindow.BackgroundImage.Dispose()
                                    m_PreviewWindow.BackgroundImageLayout = ImageLayout.Center
                                    m_PreviewWindow.BackgroundImage = GetBitmapFromHBitmap(hBitmap)
                                    DeleteObject(hBitmap)
                                    bolRet = True
                                End If
                            End If
                        End If
                    End If

                    If Marshal.ReleaseComObject(m_IPreviewHandler) = 0 Then m_IPreviewHandler = Nothing

                End If

            End If

        End If

        ' Optional falls die PreviewHandler fehlschlagen!
        If Not bolRet Then

            Dim pIShellItemImageFactory As IntPtr

            If SHCreateItemFromParsingName(m_PreviewFile, IntPtr.Zero,
                                           New Guid(IID_IShellItemImageFactory),
                                           pIShellItemImageFactory) = S_OK Then

                Dim oIShellItemImageFactory As Object = Marshal.GetObjectForIUnknown(pIShellItemImageFactory)

                If oIShellItemImageFactory IsNot Nothing Then

                    Dim hBitmap As IntPtr

                    If CType(oIShellItemImageFactory, IShellItemImageFactory).GetImage(New Size(Math.Min(PrevSize.Width, PrevSize.Height),
                                                                                                Math.Min(PrevSize.Width, PrevSize.Height)),
                                                                                       SIIGBF.SIIGBF_RESIZETOFIT, hBitmap) = S_OK Then

                        If hBitmap <> IntPtr.Zero Then
                            If m_PreviewWindow.BackgroundImage IsNot Nothing Then m_PreviewWindow.BackgroundImage.Dispose()
                            m_PreviewWindow.BackgroundImageLayout = ImageLayout.Center
                            m_PreviewWindow.BackgroundImage = GetBitmapFromHBitmap(hBitmap)
                            DeleteObject(hBitmap)
                            bolRet = True
                        End If

                    End If

                    Marshal.ReleaseComObject(oIShellItemImageFactory)

                End If

                Marshal.Release(pIShellItemImageFactory)

            End If

        End If

        Return bolRet

    End Function

    Private Function GetBitmapFromHBitmap(nativeHBitmap As IntPtr) As Bitmap
        Dim bmp As Bitmap = Bitmap.FromHbitmap(nativeHBitmap)
        If Bitmap.GetPixelFormatSize(bmp.PixelFormat) < 32 Then Return bmp
        Dim bmpData As New BitmapData
        If IsAlphaBitmap(bmp, bmpData) Then Return GetlAlphaBitmapFromBitmapData(bmpData)
        Return bmp
    End Function

    Private Function GetlAlphaBitmapFromBitmapData(bmpData As BitmapData) As Bitmap
        Return New Bitmap(bmpData.Width, bmpData.Height, bmpData.Stride, PixelFormat.Format32bppArgb, bmpData.Scan0)
    End Function

    Private Function IsAlphaBitmap(bmp As Bitmap, ByRef bmpData As BitmapData) As Boolean
        Dim bmBounds As New Rectangle(0, 0, bmp.Width, bmp.Height)
        bmpData = bmp.LockBits(bmBounds, ImageLockMode.ReadOnly, bmp.PixelFormat)
        Try
            For y As Integer = 0 To bmpData.Height - 1
                For x As Integer = 0 To bmpData.Width - 1
                    Dim pixelColor As Color = Color.FromArgb(Marshal.ReadInt32(bmpData.Scan0, (bmpData.Stride * y) + (4 * x)))
                    If pixelColor.A > 0 And pixelColor.A < 255 Then
                        Return True
                    End If
                Next
            Next
        Finally
            bmp.UnlockBits(bmpData)
        End Try
        Return False
    End Function

    Private Function DeletePreview() As Boolean

        Dim bolRet As Boolean = False

        If m_PreviewWindow IsNot Nothing Then
            If m_PreviewWindow.BackgroundImage IsNot Nothing Then
                m_PreviewWindow.BackgroundImage.Dispose()
                m_PreviewWindow.BackgroundImage = Nothing
            End If
        End If

        If m_IPreviewHandler IsNot Nothing Then
            If TypeOf m_IPreviewHandler Is IPreviewHandler Then CType(m_IPreviewHandler, IPreviewHandler).Unload()
            If Marshal.ReleaseComObject(m_IPreviewHandler) = 0 Then
                m_IPreviewHandler = Nothing
                bolRet = True
            End If
        End If

        Return bolRet

    End Function

    Private Function CreateInterface(CLSID As String, IID As String) As IntPtr
        Dim pInterface As IntPtr = IntPtr.Zero
        If CoCreateInstance(New Guid(CLSID), IntPtr.Zero, CLSCTX_LOCAL_SERVER, New Guid(IID), pInterface) = S_OK Then
            Return pInterface
        ElseIf CoCreateInstance(New Guid(CLSID), IntPtr.Zero, CLSCTX_INPROC_SERVER, New Guid(IID), pInterface) = S_OK Then
            Return pInterface
        End If
    End Function

    Private Function GetHandlerClsidFromFile(strFileName As String,
                                             strHandlerIID As String,
                                             ByRef strHandlerClsid As String) As Boolean
        Dim bolRet As Boolean = False
        Dim strFileExt As String = Path.GetExtension(strFileName).Substring(1)
        Dim strClsid As String = GetHandlerClsidFromFileExt("." & strFileExt, strHandlerIID)
        If strClsid.Length = 0 Then strClsid = GetHandlerClsidFromFileExt(strFileExt & "file", strHandlerIID)
        If strClsid.Length = 38 Then
            strHandlerClsid = strClsid
            bolRet = True
        End If
        Return bolRet
    End Function

    Private Function GetHandlerClsidFromFileExt(strFileExt As String,
                                                strHandlerIID As String) As String
        Dim intOut As Integer
        Dim strRet As String = String.Empty
        intOut = 64
        Dim sbClsid As New StringBuilder(intOut + 1)
        If AssocQueryStringW(ASSOCF.ASSOCF_NONE, ASSOCSTR.ASSOCSTR_SHELLEXTENSION,
                             strFileExt, "{" & strHandlerIID & "}", sbClsid, intOut) = S_OK Then
            If intOut > 0 Then strRet = sbClsid.ToString
        End If
        Return strRet
    End Function

    Private Function InitializePreviewHandler(oPreviewHandler As Object,
                                              strFileName As String) As Boolean
        Dim bolRet As Boolean = False
        If TypeOf oPreviewHandler Is IInitializeWithStream Then
            Dim pIStream As IntPtr
            If SHCreateStreamOnFileEx(strFileName, STGM.STGM_READ, FILE_ATTRIBUTE_NORMAL,
                                      False, IntPtr.Zero, pIStream) = S_OK Then
                If CType(oPreviewHandler, IInitializeWithStream).Initialize(pIStream, STGM.STGM_READ) = S_OK Then
                    bolRet = True
                End If
                Marshal.Release(pIStream)
            End If
        ElseIf TypeOf oPreviewHandler Is IInitializeWithFile Then
            If CType(oPreviewHandler, IInitializeWithFile).Initialize(strFileName, STGM.STGM_READ) = S_OK Then
                bolRet = True
            End If
        ElseIf TypeOf oPreviewHandler Is IInitializeWithItem Then
            Dim pIShellItem As IntPtr
            If SHCreateItemFromParsingName(strFileName, IntPtr.Zero,
                                           New Guid(IID_IShellItem), pIShellItem) = S_OK Then
                If CType(oPreviewHandler, IInitializeWithItem).Initialize(pIShellItem, STGM.STGM_READ) = S_OK Then
                    bolRet = True
                End If
                Marshal.Release(pIShellItem)
            End If
        End If
        Return bolRet
    End Function
#End Region

#Region "Public Properties"
    Public Property PreviewFile() As String
        Get
            PreviewFile = m_PreviewFile
        End Get
        Set(value As String)
            m_PreviewFile = value
        End Set
    End Property

    Public Property PreviewWindow() As Control
        Get
            PreviewWindow = m_PreviewWindow
        End Get
        Set(value As Control)
            m_PreviewWindow = value
        End Set
    End Property

    Public Property PreviewSize() As Size
        Get
            PreviewSize = m_PreviewSize
        End Get
        Set(value As Size)
            m_PreviewSize = value
        End Set
    End Property

    Public Property PreviewTextColor() As Integer
        Get
            PreviewTextColor = m_TextColor
        End Get
        Set(value As Integer)
            m_TextColor = value
        End Set
    End Property

    Public Property PreviewBackgroundColor() As Integer
        Get
            PreviewBackgroundColor = m_BackgroundColor
        End Get
        Set(value As Integer)
            m_BackgroundColor = value
        End Set
    End Property

#End Region

#Region "Interface IOleWindow"
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IOleWindow)>
    Private Interface IOleWindow
        <PreserveSig> Function GetWindow(<Out> ByRef phwnd As IntPtr) As Integer
        <PreserveSig> Function ContextSensitiveHelp(<[In], MarshalAs(UnmanagedType.Bool)> fEnterMode As Boolean) As Integer
    End Interface
#End Region

#Region "Interface IExtractImage"
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IExtractImage)>
    Private Interface IExtractImage
        <PreserveSig> Function GetLocation(<Out, MarshalAs(UnmanagedType.LPWStr)> pszPathBuffer As StringBuilder,
                                           <[In]> cch As Integer,
                                           <[In], Out> ByRef pdwPriority As IEI_FLAGS,
                                           <[In]> ByRef prgSize As Size,
                                           <[In]> dwRecClrDepth As Integer,
                                           <[In], Out> ByRef pdwFlags As IEIFLAG_FLAGS) As Integer
        <PreserveSig> Function Extract(<Out> ByRef phBmpThumbnail As IntPtr) As Integer
    End Interface
#End Region

#Region "Interface IPreviewHandler"
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IPreviewHandler)>
    Private Interface IPreviewHandler
        <PreserveSig> Function SetWindow(<[In]> hwnd As IntPtr,
                                         <[In]> ByRef prc As Rectangle) As Integer
        <PreserveSig> Function SetRect(<[In]> ByRef prc As Rectangle) As Integer
        <PreserveSig> Function DoPreview() As Integer
        <PreserveSig> Function Unload() As Integer
        <PreserveSig> Function SetFocus() As Integer
        <PreserveSig> Function QureyFocus(<Out> ByRef phwnd As IntPtr) As Integer
        <PreserveSig> Function TranslateAccelerator(<Out> ByRef pmsg As IntPtr) As Integer
    End Interface
#End Region

#Region "Interface IThumbnailProvider"
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IThumbnailProvider)>
    Private Interface IThumbnailProvider
        <PreserveSig> Function GetThumbnail(<[In]> cx As Integer,
                                            <Out> ByRef phbmp As IntPtr,
                                            <Out> ByRef pdwAlpha As WTS_ALPHATYPE) As Integer
    End Interface
#End Region

#Region "Interface IInitializeWithFile"
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IInitializeWithFile)>
    Private Interface IInitializeWithFile
        <PreserveSig> Function Initialize(<[In], MarshalAs(UnmanagedType.LPWStr)> pszFilePath As String,
                                          <[In]> grfMode As STGM) As Integer
    End Interface
#End Region

#Region "Interface IInitializeWithItem"
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IInitializeWithItem)>
    Private Interface IInitializeWithItem
        <PreserveSig> Function Initialize(<[In]> psi As IntPtr,
                                          <[In]> grfMode As STGM) As Integer
    End Interface
#End Region

#Region "Interface IInitializeWithStream"
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IInitializeWithStream)>
    Private Interface IInitializeWithStream
        <PreserveSig> Function Initialize(<[In]> pstream As IntPtr,
                                          <[In]> grfMode As STGM) As Integer
    End Interface
#End Region

#Region "Interface IPreviewHandlerVisuals"
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IPreviewHandlerVisuals)>
    Private Interface IPreviewHandlerVisuals
        <PreserveSig> Function SetBackgroundColor(<[In]> color As Integer) As Integer
        <PreserveSig> Function SetFont(<[In]> plf As IntPtr) As Integer
        <PreserveSig> Function SetTextColor(<[In]> color As Integer) As Integer
    End Interface
#End Region

#Region "Interface IShellItemImageFactory"
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IShellItemImageFactory)>
    Private Interface IShellItemImageFactory
        <PreserveSig> Function GetImage(<[In]> size As Size,
                                        <[In]> flags As SIIGBF,
                                        <Out> ByRef phbm As IntPtr) As Integer
    End Interface
#End Region

End Class
